# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .value_schema import ValueSchema

__all__ = ["Parameter"]


class Parameter(BaseModel):
    name: str

    value_schema: ValueSchema

    description: Optional[str] = None

    inferrable: Optional[bool] = None

    required: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["Requirements", "Authorization", "AuthorizationOauth2"]


class AuthorizationOauth2(BaseModel):
    scopes: Optional[List[str]] = None


class Authorization(BaseModel):
    oauth2: Optional[AuthorizationOauth2] = None

    provider_id: Optional[str] = None

    provider_type: Optional[str] = None


class Requirements(BaseModel):
    authorization: Optional[Authorization] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["HealthSchema"]


class HealthSchema(BaseModel):
    healthy: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AuthStatusParams"]


class AuthStatusParams(TypedDict, total=False):
    authorization_id: Required[Annotated[str, PropertyInfo(alias="authorizationId")]]
    """Authorization ID"""

    scopes: str
    """Scopes"""

    wait: int
    """Timeout in seconds (max 59)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AuthorizationResponse", "Context"]


class Context(BaseModel):
    token: Optional[str] = None

    user_info: Optional[Dict[str, object]] = None


class AuthorizationResponse(BaseModel):
    authorization_id: Optional[str] = None

    authorization_url: Optional[str] = None

    context: Optional[Context] = None

    scopes: Optional[List[str]] = None

    status: Optional[Literal["pending", "completed", "failed"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .chat_message import ChatMessage
from .shared.authorization_response import AuthorizationResponse

__all__ = ["Choice"]


class Choice(BaseModel):
    finish_reason: Optional[str] = None

    index: Optional[int] = None

    logprobs: Optional[object] = None

    message: Optional[ChatMessage] = None

    tool_authorizations: Optional[List[AuthorizationResponse]] = None

    tool_messages: Optional[List[ChatMessage]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ToolAuthorizeParams"]


class ToolAuthorizeParams(TypedDict, total=False):
    tool_name: Required[str]

    user_id: Required[str]

    tool_version: str
    """Optional: if not provided, any version is used"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import tool_get_params, tool_list_params, tool_execute_params, tool_authorize_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from .formatted import (
    FormattedResource,
    AsyncFormattedResource,
    FormattedResourceWithRawResponse,
    AsyncFormattedResourceWithRawResponse,
    FormattedResourceWithStreamingResponse,
    AsyncFormattedResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncOffsetPage, AsyncOffsetPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.response import Response
from ...types.shared.tool_definition import ToolDefinition
from ...types.shared.authorization_response import AuthorizationResponse

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    @cached_property
    def formatted(self) -> FormattedResource:
        return FormattedResource(self._client)

    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        toolkit: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncOffsetPage[ToolDefinition]:
        """
        Returns a page of tools, optionally filtered by toolkit

        Args:
          limit: Number of items to return (default: 25, max: 100)

          offset: Offset from the start of the list (default: 0)

          toolkit: Toolkit name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/tools/list",
            page=SyncOffsetPage[ToolDefinition],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "toolkit": toolkit,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=ToolDefinition,
        )

    def authorize(
        self,
        *,
        tool_name: str,
        user_id: str,
        tool_version: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AuthorizationResponse:
        """
        Authorizes a user for a specific tool by name

        Args:
          tool_version: Optional: if not provided, any version is used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/v1/tools/authorize",
            body=maybe_transform(
                {
                    "tool_name": tool_name,
                    "user_id": user_id,
                    "tool_version": tool_version,
                },
                tool_authorize_params.ToolAuthorizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AuthorizationResponse,
        )

    def execute(
        self,
        *,
        tool_name: str,
        inputs: object | NotGiven = NOT_GIVEN,
        tool_version: str | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Response:
        """
        Executes a tool by name and arguments

        Args:
          inputs: JSON input to the tool, if any

          tool_version: Optional: if not provided, any version is used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/v1/tools/execute",
            body=maybe_transform(
                {
                    "tool_name": tool_name,
                    "inputs": inputs,
                    "tool_version": tool_version,
                    "user_id": user_id,
                },
                tool_execute_params.ToolExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Response,
        )

    def get(
        self,
        *,
        tool_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolDefinition:
        """
        Returns the arcade tool specification for a specific tool

        Args:
          tool_id: Tool ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/tools/definition",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"tool_id": tool_id}, tool_get_params.ToolGetParams),
            ),
            cast_to=ToolDefinition,
        )


class AsyncToolsResource(AsyncAPIResource):
    @cached_property
    def formatted(self) -> AsyncFormattedResource:
        return AsyncFormattedResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        toolkit: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ToolDefinition, AsyncOffsetPage[ToolDefinition]]:
        """
        Returns a page of tools, optionally filtered by toolkit

        Args:
          limit: Number of items to return (default: 25, max: 100)

          offset: Offset from the start of the list (default: 0)

          toolkit: Toolkit name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/tools/list",
            page=AsyncOffsetPage[ToolDefinition],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "toolkit": toolkit,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=ToolDefinition,
        )

    async def authorize(
        self,
        *,
        tool_name: str,
        user_id: str,
        tool_version: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AuthorizationResponse:
        """
        Authorizes a user for a specific tool by name

        Args:
          tool_version: Optional: if not provided, any version is used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/v1/tools/authorize",
            body=await async_maybe_transform(
                {
                    "tool_name": tool_name,
                    "user_id": user_id,
                    "tool_version": tool_version,
                },
                tool_authorize_params.ToolAuthorizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AuthorizationResponse,
        )

    async def execute(
        self,
        *,
        tool_name: str,
        inputs: object | NotGiven = NOT_GIVEN,
        tool_version: str | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Response:
        """
        Executes a tool by name and arguments

        Args:
          inputs: JSON input to the tool, if any

          tool_version: Optional: if not provided, any version is used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/v1/tools/execute",
            body=await async_maybe_transform(
                {
                    "tool_name": tool_name,
                    "inputs": inputs,
                    "tool_version": tool_version,
                    "user_id": user_id,
                },
                tool_execute_params.ToolExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Response,
        )

    async def get(
        self,
        *,
        tool_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolDefinition:
        """
        Returns the arcade tool specification for a specific tool

        Args:
          tool_id: Tool ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/tools/definition",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"tool_id": tool_id}, tool_get_params.ToolGetParams),
            ),
            cast_to=ToolDefinition,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.list = to_raw_response_wrapper(
            tools.list,
        )
        self.authorize = to_raw_response_wrapper(
            tools.authorize,
        )
        self.execute = to_raw_response_wrapper(
            tools.execute,
        )
        self.get = to_raw_response_wrapper(
            tools.get,
        )

    @cached_property
    def formatted(self) -> FormattedResourceWithRawResponse:
        return FormattedResourceWithRawResponse(self._tools.formatted)


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.list = async_to_raw_response_wrapper(
            tools.list,
        )
        self.authorize = async_to_raw_response_wrapper(
            tools.authorize,
        )
        self.execute = async_to_raw_response_wrapper(
            tools.execute,
        )
        self.get = async_to_raw_response_wrapper(
            tools.get,
        )

    @cached_property
    def formatted(self) -> AsyncFormattedResourceWithRawResponse:
        return AsyncFormattedResourceWithRawResponse(self._tools.formatted)


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.list = to_streamed_response_wrapper(
            tools.list,
        )
        self.authorize = to_streamed_response_wrapper(
            tools.authorize,
        )
        self.execute = to_streamed_response_wrapper(
            tools.execute,
        )
        self.get = to_streamed_response_wrapper(
            tools.get,
        )

    @cached_property
    def formatted(self) -> FormattedResourceWithStreamingResponse:
        return FormattedResourceWithStreamingResponse(self._tools.formatted)


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.list = async_to_streamed_response_wrapper(
            tools.list,
        )
        self.authorize = async_to_streamed_response_wrapper(
            tools.authorize,
        )
        self.execute = async_to_streamed_response_wrapper(
            tools.execute,
        )
        self.get = async_to_streamed_response_wrapper(
            tools.get,
        )

    @cached_property
    def formatted(self) -> AsyncFormattedResourceWithStreamingResponse:
        return AsyncFormattedResourceWithStreamingResponse(self._tools.formatted)



from __future__ import annotations

import os
import asyncio
import logging
from typing import TYPE_CHECKING, Iterator, AsyncIterator

import pytest

from arcadepy import Arcade, AsyncArcade

if TYPE_CHECKING:
    from _pytest.fixtures import FixtureRequest

pytest.register_assert_rewrite("tests.utils")

logging.getLogger("arcadepy").setLevel(logging.DEBUG)


@pytest.fixture(scope="session")
def event_loop() -> Iterator[asyncio.AbstractEventLoop]:
    loop = asyncio.new_event_loop()
    yield loop
    loop.close()


base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")

api_key = "My API Key"


@pytest.fixture(scope="session")
def client(request: FixtureRequest) -> Iterator[Arcade]:
    strict = getattr(request, "param", True)
    if not isinstance(strict, bool):
        raise TypeError(f"Unexpected fixture parameter type {type(strict)}, expected {bool}")

    with Arcade(base_url=base_url, api_key=api_key, _strict_response_validation=strict) as client:
        yield client


@pytest.fixture(scope="session")
async def async_client(request: FixtureRequest) -> AsyncIterator[AsyncArcade]:
    strict = getattr(request, "param", True)
    if not isinstance(strict, bool):
        raise TypeError(f"Unexpected fixture parameter type {type(strict)}, expected {bool}")

    async with AsyncArcade(base_url=base_url, api_key=api_key, _strict_response_validation=strict) as client:
        yield client



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.tools import definition_get_params
from ..._base_client import make_request_options
from ...types.shared.tool_definition import ToolDefinition

__all__ = ["DefinitionResource", "AsyncDefinitionResource"]


class DefinitionResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DefinitionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return DefinitionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DefinitionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return DefinitionResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        director_id: str,
        tool_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolDefinition:
        """
        Returns the arcade tool specification for a specific tool

        Args:
          director_id: Director ID

          tool_id: Tool ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/tools/definition",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "director_id": director_id,
                        "tool_id": tool_id,
                    },
                    definition_get_params.DefinitionGetParams,
                ),
            ),
            cast_to=ToolDefinition,
        )


class AsyncDefinitionResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDefinitionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncDefinitionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDefinitionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncDefinitionResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        director_id: str,
        tool_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolDefinition:
        """
        Returns the arcade tool specification for a specific tool

        Args:
          director_id: Director ID

          tool_id: Tool ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/tools/definition",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "director_id": director_id,
                        "tool_id": tool_id,
                    },
                    definition_get_params.DefinitionGetParams,
                ),
            ),
            cast_to=ToolDefinition,
        )


class DefinitionResourceWithRawResponse:
    def __init__(self, definition: DefinitionResource) -> None:
        self._definition = definition

        self.get = to_raw_response_wrapper(
            definition.get,
        )


class AsyncDefinitionResourceWithRawResponse:
    def __init__(self, definition: AsyncDefinitionResource) -> None:
        self._definition = definition

        self.get = async_to_raw_response_wrapper(
            definition.get,
        )


class DefinitionResourceWithStreamingResponse:
    def __init__(self, definition: DefinitionResource) -> None:
        self._definition = definition

        self.get = to_streamed_response_wrapper(
            definition.get,
        )


class AsyncDefinitionResourceWithStreamingResponse:
    def __init__(self, definition: AsyncDefinitionResource) -> None:
        self._definition = definition

        self.get = async_to_streamed_response_wrapper(
            definition.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.chat import completion_create_params
from ..._base_client import make_request_options
from ...types.chat_response import ChatResponse
from ...types.chat_message_param import ChatMessageParam

__all__ = ["CompletionsResource", "AsyncCompletionsResource"]


class CompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return CompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return CompletionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        frequency_penalty: int | NotGiven = NOT_GIVEN,
        logit_bias: Dict[str, int] | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        messages: Iterable[ChatMessageParam] | NotGiven = NOT_GIVEN,
        model: str | NotGiven = NOT_GIVEN,
        n: int | NotGiven = NOT_GIVEN,
        parallel_tool_calls: bool | NotGiven = NOT_GIVEN,
        presence_penalty: int | NotGiven = NOT_GIVEN,
        response_format: Literal["json_object", "text"] | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        stop: List[str] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        stream_options: completion_create_params.StreamOptions | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        tool_choice: object | NotGiven = NOT_GIVEN,
        tools: object | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> ChatResponse:
        """
        Talk to different LLM Chat APIs via OpenAI's API

        Args:
          logit_bias: LogitBias is must be a token id string (specified by their token ID in the
              tokenizer), not a word string. incorrect: `"logit_bias":{"You": 6}`, correct:
              `"logit_bias":{"1639": 6}` refs:
              https://platform.openai.com/docs/api-reference/chat/create#chat/create-logit_bias

          logprobs: LogProbs indicates whether to return log probabilities of the output tokens or
              not. If true, returns the log probabilities of each output token returned in the
              content of message. This option is currently not available on the
              gpt-4-vision-preview model.

          parallel_tool_calls: Disable the default behavior of parallel tool calls by setting it: false.

          stream_options: Options for streaming response. Only set this when you set stream: true.

          tool_choice: This can be either a string or an ToolChoice object.

          top_logprobs: TopLogProbs is an integer between 0 and 5 specifying the number of most likely
              tokens to return at each token position, each with an associated log
              probability. logprobs must be set to true if this parameter is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/v1/chat/completions",
            body=maybe_transform(
                {
                    "frequency_penalty": frequency_penalty,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "messages": messages,
                    "model": model,
                    "n": n,
                    "parallel_tool_calls": parallel_tool_calls,
                    "presence_penalty": presence_penalty,
                    "response_format": response_format,
                    "seed": seed,
                    "stop": stop,
                    "stream": stream,
                    "stream_options": stream_options,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "user": user,
                },
                completion_create_params.CompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=ChatResponse,
        )


class AsyncCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncCompletionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        frequency_penalty: int | NotGiven = NOT_GIVEN,
        logit_bias: Dict[str, int] | NotGiven = NOT_GIVEN,
        logprobs: bool | NotGiven = NOT_GIVEN,
        max_tokens: int | NotGiven = NOT_GIVEN,
        messages: Iterable[ChatMessageParam] | NotGiven = NOT_GIVEN,
        model: str | NotGiven = NOT_GIVEN,
        n: int | NotGiven = NOT_GIVEN,
        parallel_tool_calls: bool | NotGiven = NOT_GIVEN,
        presence_penalty: int | NotGiven = NOT_GIVEN,
        response_format: Literal["json_object", "text"] | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        stop: List[str] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        stream_options: completion_create_params.StreamOptions | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        tool_choice: object | NotGiven = NOT_GIVEN,
        tools: object | NotGiven = NOT_GIVEN,
        top_logprobs: int | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> ChatResponse:
        """
        Talk to different LLM Chat APIs via OpenAI's API

        Args:
          logit_bias: LogitBias is must be a token id string (specified by their token ID in the
              tokenizer), not a word string. incorrect: `"logit_bias":{"You": 6}`, correct:
              `"logit_bias":{"1639": 6}` refs:
              https://platform.openai.com/docs/api-reference/chat/create#chat/create-logit_bias

          logprobs: LogProbs indicates whether to return log probabilities of the output tokens or
              not. If true, returns the log probabilities of each output token returned in the
              content of message. This option is currently not available on the
              gpt-4-vision-preview model.

          parallel_tool_calls: Disable the default behavior of parallel tool calls by setting it: false.

          stream_options: Options for streaming response. Only set this when you set stream: true.

          tool_choice: This can be either a string or an ToolChoice object.

          top_logprobs: TopLogProbs is an integer between 0 and 5 specifying the number of most likely
              tokens to return at each token position, each with an associated log
              probability. logprobs must be set to true if this parameter is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/v1/chat/completions",
            body=await async_maybe_transform(
                {
                    "frequency_penalty": frequency_penalty,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "messages": messages,
                    "model": model,
                    "n": n,
                    "parallel_tool_calls": parallel_tool_calls,
                    "presence_penalty": presence_penalty,
                    "response_format": response_format,
                    "seed": seed,
                    "stop": stop,
                    "stream": stream,
                    "stream_options": stream_options,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "user": user,
                },
                completion_create_params.CompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=ChatResponse,
        )


class CompletionsResourceWithRawResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_raw_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithRawResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_raw_response_wrapper(
            completions.create,
        )


class CompletionsResourceWithStreamingResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_streamed_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithStreamingResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_streamed_response_wrapper(
            completions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncOffsetPage, AsyncOffsetPage
from ...types.tools import formatted_get_params, formatted_list_params
from ..._base_client import AsyncPaginator, make_request_options

__all__ = ["FormattedResource", "AsyncFormattedResource"]


class FormattedResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FormattedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return FormattedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FormattedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return FormattedResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        format: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        toolkit: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncOffsetPage[object]:
        """
        Returns a page of tools, optionally filtered by toolkit, formatted for a
        specific provider

        Args:
          format: Provider format

          limit: Number of items to return (default: 25, max: 100)

          offset: Offset from the start of the list (default: 0)

          toolkit: Toolkit name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/tools/formatted/list",
            page=SyncOffsetPage[object],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "format": format,
                        "limit": limit,
                        "offset": offset,
                        "toolkit": toolkit,
                    },
                    formatted_list_params.FormattedListParams,
                ),
            ),
            model=object,
        )

    def get(
        self,
        *,
        tool_id: str,
        format: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Returns the formatted tool specification for a specific tool, given a provider

        Args:
          tool_id: Tool ID

          format: Provider format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/tools/formatted/definition",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "tool_id": tool_id,
                        "format": format,
                    },
                    formatted_get_params.FormattedGetParams,
                ),
            ),
            cast_to=object,
        )


class AsyncFormattedResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFormattedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncFormattedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFormattedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncFormattedResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        format: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        toolkit: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[object, AsyncOffsetPage[object]]:
        """
        Returns a page of tools, optionally filtered by toolkit, formatted for a
        specific provider

        Args:
          format: Provider format

          limit: Number of items to return (default: 25, max: 100)

          offset: Offset from the start of the list (default: 0)

          toolkit: Toolkit name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/tools/formatted/list",
            page=AsyncOffsetPage[object],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "format": format,
                        "limit": limit,
                        "offset": offset,
                        "toolkit": toolkit,
                    },
                    formatted_list_params.FormattedListParams,
                ),
            ),
            model=object,
        )

    async def get(
        self,
        *,
        tool_id: str,
        format: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Returns the formatted tool specification for a specific tool, given a provider

        Args:
          tool_id: Tool ID

          format: Provider format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/tools/formatted/definition",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "tool_id": tool_id,
                        "format": format,
                    },
                    formatted_get_params.FormattedGetParams,
                ),
            ),
            cast_to=object,
        )


class FormattedResourceWithRawResponse:
    def __init__(self, formatted: FormattedResource) -> None:
        self._formatted = formatted

        self.list = to_raw_response_wrapper(
            formatted.list,
        )
        self.get = to_raw_response_wrapper(
            formatted.get,
        )


class AsyncFormattedResourceWithRawResponse:
    def __init__(self, formatted: AsyncFormattedResource) -> None:
        self._formatted = formatted

        self.list = async_to_raw_response_wrapper(
            formatted.list,
        )
        self.get = async_to_raw_response_wrapper(
            formatted.get,
        )


class FormattedResourceWithStreamingResponse:
    def __init__(self, formatted: FormattedResource) -> None:
        self._formatted = formatted

        self.list = to_streamed_response_wrapper(
            formatted.list,
        )
        self.get = to_streamed_response_wrapper(
            formatted.get,
        )


class AsyncFormattedResourceWithStreamingResponse:
    def __init__(self, formatted: AsyncFormattedResource) -> None:
        self._formatted = formatted

        self.list = async_to_streamed_response_wrapper(
            formatted.list,
        )
        self.get = async_to_streamed_response_wrapper(
            formatted.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .auth import (
    AuthResource,
    AsyncAuthResource,
    AuthResourceWithRawResponse,
    AsyncAuthResourceWithRawResponse,
    AuthResourceWithStreamingResponse,
    AsyncAuthResourceWithStreamingResponse,
)
from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)

__all__ = [
    "AuthResource",
    "AsyncAuthResource",
    "AuthResourceWithRawResponse",
    "AsyncAuthResourceWithRawResponse",
    "AuthResourceWithStreamingResponse",
    "AsyncAuthResourceWithStreamingResponse",
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["FormattedGetParams"]


class FormattedGetParams(TypedDict, total=False):
    tool_id: Required[Annotated[str, PropertyInfo(alias="toolId")]]
    """Tool ID"""

    format: str
    """Provider format"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.authorization_response import AuthorizationResponse

__all__ = ["ResponseOutput", "Error"]


class Error(BaseModel):
    message: str

    additional_prompt_content: Optional[str] = None

    can_retry: Optional[bool] = None

    developer_message: Optional[str] = None

    retry_after_ms: Optional[int] = None


class ResponseOutput(BaseModel):
    error: Optional[Error] = None

    requires_authorization: Optional[AuthorizationResponse] = None

    value: Optional[object] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import auth_status_params, auth_authorize_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.shared.authorization_response import AuthorizationResponse

__all__ = ["AuthResource", "AsyncAuthResource"]


class AuthResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AuthResourceWithStreamingResponse(self)

    def authorize(
        self,
        *,
        auth_requirement: auth_authorize_params.AuthRequirement,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AuthorizationResponse:
        """
        Starts the authorization process for given authorization requirements

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/v1/auth/authorize",
            body=maybe_transform(
                {
                    "auth_requirement": auth_requirement,
                    "user_id": user_id,
                },
                auth_authorize_params.AuthAuthorizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AuthorizationResponse,
        )

    def status(
        self,
        *,
        authorization_id: str,
        scopes: str | NotGiven = NOT_GIVEN,
        wait: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AuthorizationResponse:
        """Checks the status of an ongoing authorization process for a specific tool.

        If
        'wait' param is present, does not respond until either the auth status becomes
        completed or the timeout is reached.

        Args:
          authorization_id: Authorization ID

          scopes: Scopes

          wait: Timeout in seconds (max 59)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/auth/status",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "authorization_id": authorization_id,
                        "scopes": scopes,
                        "wait": wait,
                    },
                    auth_status_params.AuthStatusParams,
                ),
            ),
            cast_to=AuthorizationResponse,
        )


class AsyncAuthResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncAuthResourceWithStreamingResponse(self)

    async def authorize(
        self,
        *,
        auth_requirement: auth_authorize_params.AuthRequirement,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AuthorizationResponse:
        """
        Starts the authorization process for given authorization requirements

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/v1/auth/authorize",
            body=await async_maybe_transform(
                {
                    "auth_requirement": auth_requirement,
                    "user_id": user_id,
                },
                auth_authorize_params.AuthAuthorizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AuthorizationResponse,
        )

    async def status(
        self,
        *,
        authorization_id: str,
        scopes: str | NotGiven = NOT_GIVEN,
        wait: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AuthorizationResponse:
        """Checks the status of an ongoing authorization process for a specific tool.

        If
        'wait' param is present, does not respond until either the auth status becomes
        completed or the timeout is reached.

        Args:
          authorization_id: Authorization ID

          scopes: Scopes

          wait: Timeout in seconds (max 59)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/auth/status",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "authorization_id": authorization_id,
                        "scopes": scopes,
                        "wait": wait,
                    },
                    auth_status_params.AuthStatusParams,
                ),
            ),
            cast_to=AuthorizationResponse,
        )


class AuthResourceWithRawResponse:
    def __init__(self, auth: AuthResource) -> None:
        self._auth = auth

        self.authorize = to_raw_response_wrapper(
            auth.authorize,
        )
        self.status = to_raw_response_wrapper(
            auth.status,
        )


class AsyncAuthResourceWithRawResponse:
    def __init__(self, auth: AsyncAuthResource) -> None:
        self._auth = auth

        self.authorize = async_to_raw_response_wrapper(
            auth.authorize,
        )
        self.status = async_to_raw_response_wrapper(
            auth.status,
        )


class AuthResourceWithStreamingResponse:
    def __init__(self, auth: AuthResource) -> None:
        self._auth = auth

        self.authorize = to_streamed_response_wrapper(
            auth.authorize,
        )
        self.status = to_streamed_response_wrapper(
            auth.status,
        )


class AsyncAuthResourceWithStreamingResponse:
    def __init__(self, auth: AsyncAuthResource) -> None:
        self._auth = auth

        self.authorize = async_to_streamed_response_wrapper(
            auth.authorize,
        )
        self.status = async_to_streamed_response_wrapper(
            auth.status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arcadepy import Arcade, AsyncArcade
from tests.utils import assert_matches_type
from arcadepy.types import HealthSchema

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestHealth:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_check(self, client: Arcade) -> None:
        health = client.health.check()
        assert_matches_type(HealthSchema, health, path=["response"])

    @parametrize
    def test_raw_response_check(self, client: Arcade) -> None:
        response = client.health.with_raw_response.check()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        health = response.parse()
        assert_matches_type(HealthSchema, health, path=["response"])

    @parametrize
    def test_streaming_response_check(self, client: Arcade) -> None:
        with client.health.with_streaming_response.check() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            health = response.parse()
            assert_matches_type(HealthSchema, health, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncHealth:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_check(self, async_client: AsyncArcade) -> None:
        health = await async_client.health.check()
        assert_matches_type(HealthSchema, health, path=["response"])

    @parametrize
    async def test_raw_response_check(self, async_client: AsyncArcade) -> None:
        response = await async_client.health.with_raw_response.check()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        health = await response.parse()
        assert_matches_type(HealthSchema, health, path=["response"])

    @parametrize
    async def test_streaming_response_check(self, async_client: AsyncArcade) -> None:
        async with async_client.health.with_streaming_response.check() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            health = await response.parse()
            assert_matches_type(HealthSchema, health, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["AuthAuthorizationParams", "AuthRequirement", "AuthRequirementOauth2"]


class AuthAuthorizationParams(TypedDict, total=False):
    auth_requirement: Required[AuthRequirement]

    user_id: Required[str]


class AuthRequirementOauth2(TypedDict, total=False):
    authority: str

    scopes: List[str]


class AuthRequirement(TypedDict, total=False):
    provider: Required[str]

    oauth2: AuthRequirementOauth2



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ToolkitDefinition"]


class ToolkitDefinition(BaseModel):
    name: str

    description: Optional[str] = None

    version: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .usage import Usage
from .choice import Choice
from .._models import BaseModel

__all__ = ["ChatResponse"]


class ChatResponse(BaseModel):
    id: Optional[str] = None

    choices: Optional[List[Choice]] = None

    created: Optional[int] = None

    model: Optional[str] = None

    object: Optional[str] = None

    system_fingerprint: Optional[str] = None

    usage: Optional[Usage] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ToolRetrieveParams"]


class ToolRetrieveParams(TypedDict, total=False):
    director_id: Required[str]
    """Director ID"""

    tool_id: Required[str]
    """Tool ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    @cached_property
    def completions(self) -> CompletionsResource:
        return CompletionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return ChatResourceWithStreamingResponse(self)


class AsyncChatResource(AsyncAPIResource):
    @cached_property
    def completions(self) -> AsyncCompletionsResource:
        return AsyncCompletionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncChatResourceWithStreamingResponse(self)


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

    @cached_property
    def completions(self) -> CompletionsResourceWithRawResponse:
        return CompletionsResourceWithRawResponse(self._chat.completions)


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

    @cached_property
    def completions(self) -> AsyncCompletionsResourceWithRawResponse:
        return AsyncCompletionsResourceWithRawResponse(self._chat.completions)


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

    @cached_property
    def completions(self) -> CompletionsResourceWithStreamingResponse:
        return CompletionsResourceWithStreamingResponse(self._chat.completions)


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

    @cached_property
    def completions(self) -> AsyncCompletionsResourceWithStreamingResponse:
        return AsyncCompletionsResourceWithStreamingResponse(self._chat.completions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arcadepy import Arcade, AsyncArcade
from tests.utils import assert_matches_type
from arcadepy.pagination import SyncOffsetPage, AsyncOffsetPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFormatted:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: Arcade) -> None:
        formatted = client.tools.formatted.list()
        assert_matches_type(SyncOffsetPage[object], formatted, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Arcade) -> None:
        formatted = client.tools.formatted.list(
            format="format",
            limit=0,
            offset=0,
            toolkit="toolkit",
        )
        assert_matches_type(SyncOffsetPage[object], formatted, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Arcade) -> None:
        response = client.tools.formatted.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        formatted = response.parse()
        assert_matches_type(SyncOffsetPage[object], formatted, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Arcade) -> None:
        with client.tools.formatted.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            formatted = response.parse()
            assert_matches_type(SyncOffsetPage[object], formatted, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get(self, client: Arcade) -> None:
        formatted = client.tools.formatted.get(
            tool_id="toolId",
        )
        assert_matches_type(object, formatted, path=["response"])

    @parametrize
    def test_method_get_with_all_params(self, client: Arcade) -> None:
        formatted = client.tools.formatted.get(
            tool_id="toolId",
            format="format",
        )
        assert_matches_type(object, formatted, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Arcade) -> None:
        response = client.tools.formatted.with_raw_response.get(
            tool_id="toolId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        formatted = response.parse()
        assert_matches_type(object, formatted, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Arcade) -> None:
        with client.tools.formatted.with_streaming_response.get(
            tool_id="toolId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            formatted = response.parse()
            assert_matches_type(object, formatted, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncFormatted:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_list(self, async_client: AsyncArcade) -> None:
        formatted = await async_client.tools.formatted.list()
        assert_matches_type(AsyncOffsetPage[object], formatted, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncArcade) -> None:
        formatted = await async_client.tools.formatted.list(
            format="format",
            limit=0,
            offset=0,
            toolkit="toolkit",
        )
        assert_matches_type(AsyncOffsetPage[object], formatted, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArcade) -> None:
        response = await async_client.tools.formatted.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        formatted = await response.parse()
        assert_matches_type(AsyncOffsetPage[object], formatted, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArcade) -> None:
        async with async_client.tools.formatted.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            formatted = await response.parse()
            assert_matches_type(AsyncOffsetPage[object], formatted, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get(self, async_client: AsyncArcade) -> None:
        formatted = await async_client.tools.formatted.get(
            tool_id="toolId",
        )
        assert_matches_type(object, formatted, path=["response"])

    @parametrize
    async def test_method_get_with_all_params(self, async_client: AsyncArcade) -> None:
        formatted = await async_client.tools.formatted.get(
            tool_id="toolId",
            format="format",
        )
        assert_matches_type(object, formatted, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncArcade) -> None:
        response = await async_client.tools.formatted.with_raw_response.get(
            tool_id="toolId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        formatted = await response.parse()
        assert_matches_type(object, formatted, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncArcade) -> None:
        async with async_client.tools.formatted.with_streaming_response.get(
            tool_id="toolId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            formatted = await response.parse()
            assert_matches_type(object, formatted, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Usage"]


class Usage(BaseModel):
    completion_tokens: Optional[int] = None

    prompt_tokens: Optional[int] = None

    total_tokens: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["AuthorizationResponse", "Context"]


class Context(BaseModel):
    token: Optional[str] = None


class AuthorizationResponse(BaseModel):
    authorization_id: Optional[str] = FieldInfo(alias="authorizationID", default=None)

    authorization_url: Optional[str] = FieldInfo(alias="authorizationURL", default=None)

    context: Optional[Context] = None

    scopes: Optional[List[str]] = None

    status: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ToolListParams"]


class ToolListParams(TypedDict, total=False):
    limit: int
    """Number of items to return (default: 25, max: 100)"""

    offset: int
    """Offset from the start of the list (default: 0)"""

    toolkit: str
    """Toolkit name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ToolResponse", "FinishedAt", "Output", "OutputError"]


class FinishedAt(BaseModel):
    time_time: Optional[str] = FieldInfo(alias="time.Time", default=None)


class OutputError(BaseModel):
    message: str

    additional_prompt_content: Optional[str] = None

    can_retry: Optional[bool] = None

    developer_message: Optional[str] = None

    retry_after_ms: Optional[int] = None


class Output(BaseModel):
    value: object

    error: Optional[OutputError] = None


class ToolResponse(BaseModel):
    invocation_id: str

    duration: Optional[float] = None

    finished_at: Optional[FinishedAt] = None

    output: Optional[Output] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arcadepy import Arcade, AsyncArcade
from tests.utils import assert_matches_type
from arcadepy.types import Response
from arcadepy.pagination import SyncOffsetPage, AsyncOffsetPage
from arcadepy.types.shared import ToolDefinition, AuthorizationResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTools:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: Arcade) -> None:
        tool = client.tools.list()
        assert_matches_type(SyncOffsetPage[ToolDefinition], tool, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Arcade) -> None:
        tool = client.tools.list(
            limit=0,
            offset=0,
            toolkit="toolkit",
        )
        assert_matches_type(SyncOffsetPage[ToolDefinition], tool, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Arcade) -> None:
        response = client.tools.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(SyncOffsetPage[ToolDefinition], tool, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Arcade) -> None:
        with client.tools.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(SyncOffsetPage[ToolDefinition], tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_authorize(self, client: Arcade) -> None:
        tool = client.tools.authorize(
            tool_name="tool_name",
            user_id="user_id",
        )
        assert_matches_type(AuthorizationResponse, tool, path=["response"])

    @parametrize
    def test_method_authorize_with_all_params(self, client: Arcade) -> None:
        tool = client.tools.authorize(
            tool_name="tool_name",
            user_id="user_id",
            tool_version="tool_version",
        )
        assert_matches_type(AuthorizationResponse, tool, path=["response"])

    @parametrize
    def test_raw_response_authorize(self, client: Arcade) -> None:
        response = client.tools.with_raw_response.authorize(
            tool_name="tool_name",
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(AuthorizationResponse, tool, path=["response"])

    @parametrize
    def test_streaming_response_authorize(self, client: Arcade) -> None:
        with client.tools.with_streaming_response.authorize(
            tool_name="tool_name",
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(AuthorizationResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_execute(self, client: Arcade) -> None:
        tool = client.tools.execute(
            tool_name="tool_name",
        )
        assert_matches_type(Response, tool, path=["response"])

    @parametrize
    def test_method_execute_with_all_params(self, client: Arcade) -> None:
        tool = client.tools.execute(
            tool_name="tool_name",
            inputs={},
            tool_version="tool_version",
            user_id="user_id",
        )
        assert_matches_type(Response, tool, path=["response"])

    @parametrize
    def test_raw_response_execute(self, client: Arcade) -> None:
        response = client.tools.with_raw_response.execute(
            tool_name="tool_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(Response, tool, path=["response"])

    @parametrize
    def test_streaming_response_execute(self, client: Arcade) -> None:
        with client.tools.with_streaming_response.execute(
            tool_name="tool_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(Response, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get(self, client: Arcade) -> None:
        tool = client.tools.get(
            tool_id="toolId",
        )
        assert_matches_type(ToolDefinition, tool, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Arcade) -> None:
        response = client.tools.with_raw_response.get(
            tool_id="toolId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(ToolDefinition, tool, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Arcade) -> None:
        with client.tools.with_streaming_response.get(
            tool_id="toolId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(ToolDefinition, tool, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTools:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_list(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.list()
        assert_matches_type(AsyncOffsetPage[ToolDefinition], tool, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.list(
            limit=0,
            offset=0,
            toolkit="toolkit",
        )
        assert_matches_type(AsyncOffsetPage[ToolDefinition], tool, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArcade) -> None:
        response = await async_client.tools.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(AsyncOffsetPage[ToolDefinition], tool, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArcade) -> None:
        async with async_client.tools.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(AsyncOffsetPage[ToolDefinition], tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_authorize(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.authorize(
            tool_name="tool_name",
            user_id="user_id",
        )
        assert_matches_type(AuthorizationResponse, tool, path=["response"])

    @parametrize
    async def test_method_authorize_with_all_params(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.authorize(
            tool_name="tool_name",
            user_id="user_id",
            tool_version="tool_version",
        )
        assert_matches_type(AuthorizationResponse, tool, path=["response"])

    @parametrize
    async def test_raw_response_authorize(self, async_client: AsyncArcade) -> None:
        response = await async_client.tools.with_raw_response.authorize(
            tool_name="tool_name",
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(AuthorizationResponse, tool, path=["response"])

    @parametrize
    async def test_streaming_response_authorize(self, async_client: AsyncArcade) -> None:
        async with async_client.tools.with_streaming_response.authorize(
            tool_name="tool_name",
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(AuthorizationResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_execute(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.execute(
            tool_name="tool_name",
        )
        assert_matches_type(Response, tool, path=["response"])

    @parametrize
    async def test_method_execute_with_all_params(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.execute(
            tool_name="tool_name",
            inputs={},
            tool_version="tool_version",
            user_id="user_id",
        )
        assert_matches_type(Response, tool, path=["response"])

    @parametrize
    async def test_raw_response_execute(self, async_client: AsyncArcade) -> None:
        response = await async_client.tools.with_raw_response.execute(
            tool_name="tool_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(Response, tool, path=["response"])

    @parametrize
    async def test_streaming_response_execute(self, async_client: AsyncArcade) -> None:
        async with async_client.tools.with_streaming_response.execute(
            tool_name="tool_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(Response, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get(self, async_client: AsyncArcade) -> None:
        tool = await async_client.tools.get(
            tool_id="toolId",
        )
        assert_matches_type(ToolDefinition, tool, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncArcade) -> None:
        response = await async_client.tools.with_raw_response.get(
            tool_id="toolId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(ToolDefinition, tool, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncArcade) -> None:
        async with async_client.tools.with_streaming_response.get(
            tool_id="toolId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(ToolDefinition, tool, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ChatMessage", "ToolCall", "ToolCallFunction"]


class ToolCallFunction(BaseModel):
    arguments: Optional[str] = None

    name: Optional[str] = None


class ToolCall(BaseModel):
    id: Optional[str] = None

    function: Optional[ToolCallFunction] = None

    type: Optional[Literal["function"]] = None


class ChatMessage(BaseModel):
    content: str
    """The content of the message."""

    role: str
    """The role of the author of this message.

    One of system, user, tool, or assistant.
    """

    name: Optional[str] = None
    """tool Name"""

    tool_call_id: Optional[str] = None
    """tool_call_id"""

    tool_calls: Optional[List[ToolCall]] = None
    """tool calls if any"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ToolExecuteParams"]


class ToolExecuteParams(TypedDict, total=False):
    tool_name: Required[str]

    inputs: object
    """JSON input to the tool, if any"""

    tool_version: str
    """Optional: if not provided, any version is used"""

    user_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["AuthAuthorizeParams", "AuthRequirement", "AuthRequirementOauth2"]


class AuthAuthorizeParams(TypedDict, total=False):
    auth_requirement: Required[AuthRequirement]

    user_id: Required[str]


class AuthRequirementOauth2(TypedDict, total=False):
    scopes: List[str]


class AuthRequirement(TypedDict, total=False):
    oauth2: AuthRequirementOauth2

    provider_id: str

    provider_type: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arcadepy import Arcade, AsyncArcade
from tests.utils import assert_matches_type
from arcadepy.types.shared import AuthorizationResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuth:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_authorize(self, client: Arcade) -> None:
        auth = client.auth.authorize(
            auth_requirement={},
            user_id="user_id",
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    def test_method_authorize_with_all_params(self, client: Arcade) -> None:
        auth = client.auth.authorize(
            auth_requirement={
                "oauth2": {"scopes": ["string", "string", "string"]},
                "provider_id": "provider_id",
                "provider_type": "provider_type",
            },
            user_id="user_id",
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    def test_raw_response_authorize(self, client: Arcade) -> None:
        response = client.auth.with_raw_response.authorize(
            auth_requirement={},
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = response.parse()
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    def test_streaming_response_authorize(self, client: Arcade) -> None:
        with client.auth.with_streaming_response.authorize(
            auth_requirement={},
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = response.parse()
            assert_matches_type(AuthorizationResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_status(self, client: Arcade) -> None:
        auth = client.auth.status(
            authorization_id="authorizationId",
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    def test_method_status_with_all_params(self, client: Arcade) -> None:
        auth = client.auth.status(
            authorization_id="authorizationId",
            scopes="scopes",
            wait=0,
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    def test_raw_response_status(self, client: Arcade) -> None:
        response = client.auth.with_raw_response.status(
            authorization_id="authorizationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = response.parse()
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    def test_streaming_response_status(self, client: Arcade) -> None:
        with client.auth.with_streaming_response.status(
            authorization_id="authorizationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = response.parse()
            assert_matches_type(AuthorizationResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAuth:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_authorize(self, async_client: AsyncArcade) -> None:
        auth = await async_client.auth.authorize(
            auth_requirement={},
            user_id="user_id",
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    async def test_method_authorize_with_all_params(self, async_client: AsyncArcade) -> None:
        auth = await async_client.auth.authorize(
            auth_requirement={
                "oauth2": {"scopes": ["string", "string", "string"]},
                "provider_id": "provider_id",
                "provider_type": "provider_type",
            },
            user_id="user_id",
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    async def test_raw_response_authorize(self, async_client: AsyncArcade) -> None:
        response = await async_client.auth.with_raw_response.authorize(
            auth_requirement={},
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = await response.parse()
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    async def test_streaming_response_authorize(self, async_client: AsyncArcade) -> None:
        async with async_client.auth.with_streaming_response.authorize(
            auth_requirement={},
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = await response.parse()
            assert_matches_type(AuthorizationResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_status(self, async_client: AsyncArcade) -> None:
        auth = await async_client.auth.status(
            authorization_id="authorizationId",
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    async def test_method_status_with_all_params(self, async_client: AsyncArcade) -> None:
        auth = await async_client.auth.status(
            authorization_id="authorizationId",
            scopes="scopes",
            wait=0,
        )
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    async def test_raw_response_status(self, async_client: AsyncArcade) -> None:
        response = await async_client.auth.with_raw_response.status(
            authorization_id="authorizationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = await response.parse()
        assert_matches_type(AuthorizationResponse, auth, path=["response"])

    @parametrize
    async def test_streaming_response_status(self, async_client: AsyncArcade) -> None:
        async with async_client.auth.with_streaming_response.status(
            authorization_id="authorizationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = await response.parse()
            assert_matches_type(AuthorizationResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .formatted_get_params import FormattedGetParams as FormattedGetParams
from .formatted_list_params import FormattedListParams as FormattedListParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .error import Error as Error
from .tool_definition import ToolDefinition as ToolDefinition
from .authorization_response import AuthorizationResponse as AuthorizationResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .response_output import ResponseOutput

__all__ = ["Response", "FinishedAt"]


class FinishedAt(BaseModel):
    time_time: Optional[str] = FieldInfo(alias="time.Time", default=None)


class Response(BaseModel):
    invocation_id: str

    duration: Optional[float] = None

    finished_at: Optional[FinishedAt] = None

    output: Optional[ResponseOutput] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .parameter import Parameter

__all__ = ["Inputs"]


class Inputs(BaseModel):
    parameters: Optional[List[Parameter]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ValueSchema"]


class ValueSchema(BaseModel):
    val_type: str

    enum: Optional[List[str]] = None

    inner_val_type: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .value_schema import ValueSchema

__all__ = ["Output"]


class Output(BaseModel):
    available_modes: Optional[List[str]] = None

    description: Optional[str] = None

    value_schema: Optional[ValueSchema] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .usage import Usage as Usage
from .choice import Choice as Choice
from .inputs import Inputs as Inputs
from .output import Output as Output
from .shared import Error as Error, ToolDefinition as ToolDefinition, AuthorizationResponse as AuthorizationResponse
from .response import Response as Response
from .parameter import Parameter as Parameter
from .chat_message import ChatMessage as ChatMessage
from .requirements import Requirements as Requirements
from .value_schema import ValueSchema as ValueSchema
from .chat_response import ChatResponse as ChatResponse
from .health_schema import HealthSchema as HealthSchema
from .response_output import ResponseOutput as ResponseOutput
from .tool_get_params import ToolGetParams as ToolGetParams
from .tool_list_params import ToolListParams as ToolListParams
from .auth_status_params import AuthStatusParams as AuthStatusParams
from .chat_message_param import ChatMessageParam as ChatMessageParam
from .toolkit_definition import ToolkitDefinition as ToolkitDefinition
from .tool_execute_params import ToolExecuteParams as ToolExecuteParams
from .auth_authorize_params import AuthAuthorizeParams as AuthAuthorizeParams
from .tool_authorize_params import ToolAuthorizeParams as ToolAuthorizeParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..inputs import Inputs
from ..output import Output
from ..._models import BaseModel
from ..requirements import Requirements
from ..toolkit_definition import ToolkitDefinition

__all__ = ["ToolDefinition"]


class ToolDefinition(BaseModel):
    inputs: Inputs

    name: str

    toolkit: ToolkitDefinition

    description: Optional[str] = None

    output: Optional[Output] = None

    requirements: Optional[Requirements] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .formatted import (
    FormattedResource,
    AsyncFormattedResource,
    FormattedResourceWithRawResponse,
    AsyncFormattedResourceWithRawResponse,
    FormattedResourceWithStreamingResponse,
    AsyncFormattedResourceWithStreamingResponse,
)

__all__ = [
    "FormattedResource",
    "AsyncFormattedResource",
    "FormattedResourceWithRawResponse",
    "AsyncFormattedResourceWithRawResponse",
    "FormattedResourceWithStreamingResponse",
    "AsyncFormattedResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = [
    "ToolDefinition",
    "Inputs",
    "InputsParameter",
    "InputsParameterValueSchema",
    "Output",
    "OutputValueSchema",
    "Requirements",
    "RequirementsAuthorization",
    "RequirementsAuthorizationOauth2",
]


class InputsParameterValueSchema(BaseModel):
    val_type: str

    enum: Optional[List[str]] = None

    inner_val_type: Optional[str] = None


class InputsParameter(BaseModel):
    name: str

    value_schema: InputsParameterValueSchema

    description: Optional[str] = None

    inferrable: Optional[bool] = None

    required: Optional[bool] = None


class Inputs(BaseModel):
    parameters: Optional[List[InputsParameter]] = None


class OutputValueSchema(BaseModel):
    val_type: str

    enum: Optional[List[str]] = None

    inner_val_type: Optional[str] = None


class Output(BaseModel):
    available_modes: Optional[List[str]] = None

    description: Optional[str] = None

    value_schema: Optional[OutputValueSchema] = None


class RequirementsAuthorizationOauth2(BaseModel):
    authority: Optional[str] = None

    scopes: Optional[List[str]] = None


class RequirementsAuthorization(BaseModel):
    provider: str

    oauth2: Optional[RequirementsAuthorizationOauth2] = None


class Requirements(BaseModel):
    authorization: Optional[RequirementsAuthorization] = None


class ToolDefinition(BaseModel):
    inputs: Inputs

    name: str

    version: str

    description: Optional[str] = None

    output: Optional[Output] = None

    requirements: Optional[Requirements] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .shared.tool_definition import ToolDefinition

__all__ = ["ToolListResponse"]

ToolListResponse: TypeAlias = List[ToolDefinition]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatMessageParam", "ToolCall", "ToolCallFunction"]


class ToolCallFunction(TypedDict, total=False):
    arguments: str

    name: str


class ToolCall(TypedDict, total=False):
    id: str

    function: ToolCallFunction

    type: Literal["function"]


class ChatMessageParam(TypedDict, total=False):
    content: Required[str]
    """The content of the message."""

    role: Required[str]
    """The role of the author of this message.

    One of system, user, tool, or assistant.
    """

    name: str
    """tool Name"""

    tool_call_id: str
    """tool_call_id"""

    tool_calls: Iterable[ToolCall]
    """tool calls if any"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable
from typing_extensions import Literal, TypedDict

from ..chat_message_param import ChatMessageParam

__all__ = ["CompletionCompletionsParams", "StreamOptions"]


class CompletionCompletionsParams(TypedDict, total=False):
    frequency_penalty: int

    logit_bias: Dict[str, int]
    """
    LogitBias is must be a token id string (specified by their token ID in the
    tokenizer), not a word string. incorrect: `"logit_bias":{"You": 6}`, correct:
    `"logit_bias":{"1639": 6}` refs:
    https://platform.openai.com/docs/api-reference/chat/create#chat/create-logit_bias
    """

    logprobs: bool
    """
    LogProbs indicates whether to return log probabilities of the output tokens or
    not. If true, returns the log probabilities of each output token returned in the
    content of message. This option is currently not available on the
    gpt-4-vision-preview model.
    """

    max_tokens: int

    messages: Iterable[ChatMessageParam]

    model: str

    n: int

    parallel_tool_calls: object
    """Disable the default behavior of parallel tool calls by setting it: false."""

    presence_penalty: int

    response_format: Literal["json_object", "text"]

    seed: int

    stop: List[str]

    stream: bool

    stream_options: StreamOptions
    """Options for streaming response. Only set this when you set stream: true."""

    temperature: float

    tool_choice: object
    """This can be either a string or an ToolChoice object."""

    tools: object

    top_logprobs: int
    """
    TopLogProbs is an integer between 0 and 5 specifying the number of most likely
    tokens to return at each token position, each with an associated log
    probability. logprobs must be set to true if this parameter is used.
    """

    top_p: float

    user: str


class StreamOptions(TypedDict, total=False):
    include_usage: bool
    """
    If set, an additional chunk will be streamed before the data: [DONE] message.
    The usage field on this chunk shows the token usage statistics for the entire
    request, and the choices field will always be an empty array. All other chunks
    will also include a usage field, but with a null value.
    """
